@@ -26,26 +26,8 @@ from pydan import run
 if(sys.getfilesystemencoding()=="ascii" and sys.getdefaultencoding()=="utf-8"):
 	sys.stdout = open(sys.stdout.fileno(), mode='w', encoding='utf8', buffering=1)
 
-c={
-	"none":"\033[0m",
-	"header":"\033[38;5;166m",  # orange
-	"group":"\033[1;34m",       # light blue
-	"item":"\033[0;36m",        # cian
-	"null":"\033[1;30m",        # gray
-	"date":"\033[38;5;105m",    # xviolet
-	"str":"\033[0;32m",         # green
-	"int":"\033[1;32m",         # light green
-	"float":"\033[38;5;112m",   # xolive
-	"bool":"\033[1;36m",        # white
-	"unk":"\033[0;31m",         # red
-	"varint":"\033[1;35m",      # magentab
-	"varenv":"\033[0;35m",      # magenta
-	"varcmd":"\033[1;31m",      # red
-	"err":"\033[1;31m",         # light red
-	"important":"\033[1;33m",   # light red
-}
-
 ansi256color={
+	"none":"\x1b[0m",
 	"black":"\x1b[38;5;235m",
 	"gray":"\x1b[38;5;240m",
 	"white":"\x1b[38;5;247m",
@@ -71,6 +53,26 @@ ansi256color={
 	"magenta":"\x1b[38;5;199m",
 }
 
+c={
+	"none":ansi256color["none"],
+	"header":ansi256color["orange"],
+	"group":ansi256color["lightblue"],
+	"item":ansi256color["cyan"],
+	"null":ansi256color["gray"],
+	"date":ansi256color["violet"],
+	"str":ansi256color["green"],
+	"int":ansi256color["lightgreen"],
+	"float":ansi256color["olive"],
+	"bool":ansi256color["white"],
+	"unk":ansi256color["red"],
+	"varint":ansi256color["lightpink"],
+	"varenv":ansi256color["pink"],
+	"varcmd":ansi256color["red"],
+	"err":ansi256color["lightred"],
+	"binary":ansi256color["purple"],
+}
+
+
 def colprint(d,header=None,key=None,indent="",crop=True,format='tree',fields=None,fieldcolors=None):
 	if format=="tree":
 		colprint_tree(d,header,key,indent,crop)
@@ -100,11 +102,12 @@ def colprint_column(data,header=None,fields=None,fieldcolors=None):
 
 	for d in data:
 		for var in fields:
-			data=str(d.get(var))
-			if data is None: data="-"
+			f=d.get(var)
+			if f is None: f="·"
+			strf=str(f)
 			print(ansi256color.get(fieldcolors[var]), end="")
-			print(data,end="")
-			for f in range(0,sizes[var]-len(data)+1):
+			print(strf,end="")
+			for f in range(0,sizes[var]-len(strf)+1):
 				print(" ",end="")
 		print("\x1b[0m")
 
@@ -126,11 +129,11 @@ def colprint_tree(d,header=None,key=None,indent="",crop=True):
 	if(header): print(c["header"]+header+c["none"]);indent=indent+indentchars
 
 	# Si es diccionario iteramos
-	if(type(d)==dict or type(d)==collections.OrderedDict):  # or type(d)==CommentedMap):
+	if isinstance(d,dict) or isinstance(d,collections.OrderedDict):  # or type(d)==CommentedMap):
 	#if(type(d)==dict or type(d)==collections.OrderedDict or type(d)==CommentedMap):
 	#if hasattr(d,'__iter__') and type(d)!=str:
-		if key is not None: print(indent+c["group"]+key+":"+c["none"]); indent+=indentchars
-		if len(d)==0: print(indent+c["null"]+"empty"+c["none"])
+		if key is not None: print(f"{indent}{c['group']}{key}:{c['none']}"); indent+=indentchars
+		if len(d)==0: print(f"{indent}{c['null']}empty{c['none']}")
 		else:
 			for k in d: colprint_tree(d[k], key=k, indent=indent)
 		return
@@ -163,7 +166,7 @@ def colprint_tree(d,header=None,key=None,indent="",crop=True):
 	#		#print(k);
 	#	return
 
-	if type(d)==list or type(d)==tuple:
+	if isinstance(d,list) or isinstance(d,tuple):
 		# Vacia
 		if len(d)==0:
 			if(key is not None):
@@ -214,8 +217,8 @@ def colprint_tree(d,header=None,key=None,indent="",crop=True):
 	else: print(indent, end='')
 
 	# Mostramos valor
-	t=type(d)
-	if(t==str):
+	#t=type(d)
+	if isinstance(d,str):
 		p=re.compile("\$([a-zA-Z0-9_]*)\$")
 		vars=set(p.findall(d))
 		for k in vars: d=re.sub("\$"+k+"\$", c["varenv"]+"$"+k+"$"+c["str"], d)
@@ -226,29 +229,29 @@ def colprint_tree(d,header=None,key=None,indent="",crop=True):
 		vars=set(p.findall(d))
 		for k in vars: d=re.sub("\(\(.*\)\)", c["varcmd"]+"(("+k+"))"+c["str"], d)
 		if crop and len(d)>4000:
-			print(c["important"]+"###DATA-"+str(len(d))+"###"+c["none"])
+			print(c["binary"]+"###DATA-"+str(len(d))+"###"+c["none"])
 		else:
 			print(c["str"]+d+c["none"])
-	elif(t==datetime):
+	elif isinstance(d,datetime):
 		dtstr=re.sub("\+00:00", "Z", d.isoformat())
 		#dtstr=d.__str__()
 		print(c["date"]+dtstr+c["none"])
-	elif(t==int):
+	elif isinstance(d,int):
 		print(c["int"]+str(d)+c["none"])
-	elif(t==float):
+	elif isinstance(d,float):
 		print(c["float"]+str(d)+c["none"])
-	elif(t==bool):
+	elif isinstance(d,bool):
 		print(c["bool"]+str(d)+c["none"])
-#	elif(t==list):
-#		print
-#		for i in d:
-#			colprint_tree(i,header="x",indent=indent)
-#			#print(indent+indentchars+c["str"]+i)
-#			#print(d)
-	elif(t==type(None)):
+	#	elif(t==list):
+	#		print
+	#		for i in d:
+	#			colprint_tree(i,header="x",indent=indent)
+	#			#print(indent+indentchars+c["str"]+i)
+	#			#print(d)
+	elif d is None:
 		print(c["null"]+"null"+c["none"])
 	else:
-		print(c["err"]+"¿"+str(t.__name__)+"?"+c["none"])
+		print(f"{c['err']}¿{str(d.__name__)}?{c['none']}")
 #}}}
 
 # Por cada item en d reemplaza $var$ por el valor de la variable en la lista varlist
@@ -580,6 +583,37 @@ def contained(d1, d2):
 			if d1[k]!=d2[k]: return False
 	return True
 
+# csv tools
+def tocsv(data,sep=None,fields=None):
+
+	if sep is None: sep='|'
+	if type(sep) == int: sep='|'
+	out=""
+
+	if fields is None:
+		fields=[]
+		for row in data:
+			for k in row:
+				if k not in fields:
+					fields.append(k)
+
+	first=True
+	for var in fields:
+		if first: first=False
+		else: out=out+sep
+		out=out+var
+
+	for row in data:
+		out=out+"\n"
+		first=True
+		for var in fields:
+			if first: first=False
+			else: out=out+sep
+			val=row.get(var)
+			strval=str(val).replace("\n", "\\n")
+			if val is None: val=""
+			out=out+strval
+	return out
 
 #{{{ xml tools
 
